//
// Text rendering
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef Guit_TextImpl_hpp
#define Guit_TextImpl_hpp
#include <list>
#include <ggraphics.hpp>
#include <gfont.hpp>
#include <gicon.hpp>
namespace guit {


class GTextCache {
public:
  struct Row {
    Row(float y, float h, size_t start) : y_(y), h_(h), start_(start) {}
    float y_{}, h_{};
    size_t start_{}, length_{};
  };
  ~GTextCache() {delete rows_;}
  std::vector<Row> *rows_{};
};


class GTextRender {
public:
  GTextRender(GFontValue const&, bool richtext, GRgba const* color,
              size_t selbegin, size_t selend, GTextAttributes const*);

  GDim getSize(GString const& text);
  void draw(GGraphics&, float x, float y, GString const& text, size_t offset);

  // lays out and return the size of multiline text.
  static GDim layoutText(GFontValue const&, bool richtext, GString const& text,
                         float maxwidth, GTextAttributes const*,
                         GTextCache* = {});
private:
  int  attrFound(GString const& text, size_t offset);
  void drawPart(GGraphics&, float x, float y, GString const& text,
                bool drawlast, bool insel);
  bool findCurrentAttr(size_t offset);
  bool processAttr(GGraphics*, float x, float y, int delim,
                   GString const& attr, size_t offset);
  void processAttrBegin(GGraphics*, float x, float y, GPropList const&) ;
  void processAttrBegin(GGraphics*, float x, float y, GString const& attr);
  void processAttrEnd();
  void processImage(GGraphics*, float x, float y, size_t from, GString const& attr);

  GDim textsize_;
  bool htmltags_; 
  size_t pos_, begpart_, selbegin_, selend_;
  GTextAttributes const* attrlist_{};
  GTextAttr const* curattr_{};
  size_t curattrpos_{}; //, offset_{};

  struct StackNode {
    StackNode(GFontValue const& font, GRgba const& color, GRgba const& bg)
    : font_(font), color_(color), bgcolor_(bg) {}
    GFontValue font_;
    GRgba color_, bgcolor_;
  };
  std::vector<StackNode> stack_;
  StackNode* current_;
};

}
#endif
