//
//  MenuBar, ToolBar, StatusBar
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gevent.hpp>
#include <gtrigger.hpp>
#include <gbar.hpp>
#include <gitem.hpp>
#include <gmenu.hpp>
#include <gtext.hpp>
#include <gstring.hpp>
#include <gcontrol.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_GADGET(MenuBar)
GIMPLEMENT_GADGET(ToolBar)
GIMPLEMENT_GADGET(StatusBar)


GMenuBar::GMenuBar(GadgetType& t, GString const& spec)
: GButtonBox(t, spec) {
  modes_.stealTitle = 1;
}

GMenuBar::GMenuBar(GString const& spec)
: GButtonBox(Type(), spec) {
  modes_.stealTitle = 1;
}

void GMenuBar::add(GMenu* menu) {
  if (!menu) {
    error("add","Can't add null child");
    return;
  }
  // the title of child will the name of menubar item (if defined)
  //auto* t = menu->findProp<GTitle>();
  //GString tabtitle = t ? t->value() : "";

  auto* t = menu->findChild<GTitle>();
  GString tabtitle = t ? t->text() : "";

  if (strblank(tabtitle)) GButtonBox::add(menu);
  else *this << (*new GItem(tabtitle) << menu);
}


GToolBar::GToolBar(GadgetType& t, GString const& spec)
: GButtonBox(t, spec) {}

GToolBar::GToolBar(GString const& spec)
: GButtonBox(Type(), spec) {}



GStatusBar::GStatusBar(GadgetType& t, GString const& spec)
: GBox(t, spec) {}

GStatusBar::GStatusBar(GString const& spec)
: GBox(Type(), spec) {}


Gadget& GStatusBar::createDefaultItem(GString const& arg) {
  return *new GLabel(arg);
}

}
