//
//  Boxes that stack gadgets.
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#include <gevent.hpp>
#include <gstackbox.hpp>
#include <gitem.hpp>
#include <galign.hpp>
#include <gcontrol.hpp>
#include <gstring.hpp>
using namespace std;
namespace guit {

GIMPLEMENT_GADGET(StackBox)
GIMPLEMENT_GADGET(TabBox)

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

GStackBox::GStackBox(GadgetType& t, GString const& spec)
: GChoiceBox(t, spec) {}

GStackBox::GStackBox(GString const& spec)
: GChoiceBox(Type(), spec) {}

GStackBox& GStackBox::clone(GClone const& c) {
  auto* g = new GStackBox();
  // if (c & GClone::ShareProps) g->choice_ = choice_; done initClone()
  g->initClone(*this, c);
  return *g;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

GTabBox::~GTabBox() = default;  // avoids haeder inclusion

GTabBox::GTabBox(GadgetType& t, GString const& spec) :
GStackBox(t, spec),
tabs_(ChoiceBox()) {
  modes_.stealTitle = 1;
  tabs_ << choice_;
  *tabs_ << GFlow::row << new GAlign("c c") << choice_;
  //borderAdd(tabs_, Place::ToolBar);
  add(tabs_, (int)Place::ToolBar);
}

GTabBox::GTabBox(GString const& spec)
: GTabBox(Type(), spec) {}

GTabBox& GTabBox::clone(GClone const& c) {
  auto* g = new GTabBox;
  if (c & GClone::ShareProps) {
    //g->choice_ = choice_;  done initClone()
    *g->tabs_ << choice_;
  }
  g->initClone(*this, c);
  return *g;
}

GChoiceBox& GTabBox::tabs() {
  return *tabs_;
}

void GTabBox::setChoiceImpl(GChoice& c, bool add_to_list) {
  GChoiceBox::setChoiceImpl(c, add_to_list);
  *tabs_ << c;
}

void GTabBox::add(Gadget* child, int pos) {
  if (!child) {
    error("add","Can't add null child");
  }
  else if (child == this) {
    error("add","Can't add gadget to itself");
  }
  else if (pos < -1) {  // border area
    borderAdd(*child, (Place)pos);
  }
  else if (child->layout_.activePos) {
    GBox::add(child, pos);  // marche pas, ix decale !!
  }
  else {
    // the title of child will the name of the tab (if defined)
    GString tabtitle;
    //if (auto* t = child->findProp<GTitle>()) tabtitle = t->value();
    if (auto* box = child->toBox()) {
      if (auto* t = box->findChild<GTitle>(true)) {
        //cerr << "tabtitle " << tabtitle << endl;
        tabtitle = t->text();
        box->remove(t);
      }
    }
    // if no title, just add number
    if (strblank(tabtitle)) {
      int num = (pos >= 0 ? pos : childCount()) + 1;
      tabtitle = "  " + std::to_string(num) + "  ";
    }
    GBox::add(child, pos);
    tabs_->add(new GChoiceItem(tabtitle), pos);
  }
}

}
