//
// Margins of a gadget
//
// Guit GUI Toolkit (https://www.telecom-paris.fr/~elc/guit/)
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com) - Telecom Paris
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef GuitMargin_hpp
#define GuitMargin_hpp
#include <gvarprop.hpp>
namespace guit {

class GMarginValue {
public:
  GMarginValue();
  GMarginValue(float left, float right, float top, float bottom);

  bool operator==(GMarginValue const&) const;
  bool operator!=(GMarginValue const&) const;

  float left, right, top, bottom;
};

inline bool gconvert(GMarginValue const& from, GMarginValue& to) {to = from; return true;}
bool gconvert(GString const& from, GMarginValue& to);
bool gconvert(GMarginValue const& from, GString& to);


/// Margins of a gadget
class GMargin : public GVarProp_<GMarginValue> {
public:
  using Super = GVarProp_<GMarginValue>;

  GMargin();

  /// Constr.
  /// argument can contain 2 or 4 floats separated by spaces, i.e.:
  /// - left/right and top/bottom margins (2 floats)
  /// - left right top bottom margins (4 floats)
  explicit GMargin(GString const& value);

  /// specifies size of left/right and top/bottom margins.
  GMargin(float left_right, float top_bottom);

  /// specifies size of each margin.
  GMargin(float left, float right, float top, float bottom);

  explicit GMargin(GMarginValue const&);

  GMargin* clone(bool copy_value) const override;

  float left()   const {return value_.left;}
  float right()  const {return value_.right;}
  float top()    const {return value_.top;}
  float bottom() const {return value_.bottom;}

  using GType = GPropType_<GMargin,GProp>;
  static GType& Type();
  GType& type() const override {return Type();}

protected:
  friend class GRender;
  void updateGadget(Gadget&) override;
  void apply(Gadget*, GRender&, Specif) override;
};

/// Returns a GMargin created in the heap.
/// @{
inline GMargin& Margin() {return *new GMargin();}
inline GMargin& Margin(GString const& value) {return *new GMargin(value);}
/// @}

}
#endif
